package com.fzm.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.entity.Admin;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.AdminMapper;
import com.fzm.common.service.AdminService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author tangtuo
 * @date 2021/7/5 15:09
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AdminServiceImpl extends ServiceImpl<AdminMapper, Admin> implements AdminService {

    private final AdminMapper AdminMapper;
    private final PasswordEncoder passwordEncoder;

    @Override
    public Admin login(String username, String password) {
        QueryWrapper<Admin> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("username", username);
        Admin admin = getOne(queryWrapper);
        if (admin == null) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, "您还没有注册");
        }
        if (!passwordEncoder.matches(password, admin.getPassword())) {
            throw GlobalException.newException(ResultCode.UNAUTHORIZED, "密码有误");
        }

        Admin updateAdmin = new Admin();
        updateAdmin.setUsername(username);
        updateAdmin.setLastLoginTime(new Date());
        AdminMapper.updateByUserNameIf(updateAdmin);

        return admin;
    }

    @Override
    public PageInfo<Admin> pages(Integer pageNum, Integer pageSize, String username, String nickname, Integer roleId) {
        PageHelper.startPage(pageNum, pageSize);
        List<Admin> list = AdminMapper.listAdmin(username, nickname, roleId);
        return new PageInfo<>(list);
    }

    @Override
    public void updateByUserNameIf(Admin admin) {
        AdminMapper.updateByUserNameIf(admin);
    }

    @Override
    public Admin selectByUsername(String username) {
        QueryWrapper<Admin> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("username", username);
        return getOne(queryWrapper);
    }
}
