package com.fzm.portal.config;

import org.springframework.amqp.core.*;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author tangtuo
 * @date 2022/1/21 11:28
 */
@Configuration
public class RabbitConfig {

    /**
     * 处理订单的交换机
     */
    @Bean
    public DirectExchange orderExchange() {
        return new DirectExchange("order-exchange", true, false);
    }

    /**
     * 处理订单的队列
     *
     * @return
     */
    @Bean
    public Queue orderQueue() {
        return new Queue("order.process.queue", true);
    }

    /**
     * 处理订单的队列和交换机之间的绑定
     *
     * @return
     */
    @Bean
    public Binding orderBinding() {
        return BindingBuilder.bind(orderQueue()).to(orderExchange()).with("order.process");
    }


    /**
     * 发行nft的交换机
     *
     * @return
     */
    @Bean
    public DirectExchange nftExchange() {
        return new DirectExchange("nft-exchange", true, false);
    }

    /**
     * 死信交换机,死信到期后将会被转发到这个交换机,然后再路由到相应的队列
     */
    @Bean
    public DirectExchange dlNftExchange() {
        return new DirectExchange("dl-nft-exchange", true, false);
    }

    /**
     * 死信队列,用户把nft的交易hash发到此队列,
     *
     * @return
     */
    @Bean
    public Queue dlNftQueue() {
        return QueueBuilder.durable("dl.nft.queue")
                .ttl(1000 * 10)
                .deadLetterExchange("dl-nft-exchange")
                .deadLetterRoutingKey("nft")
                .build();
    }

    /**
     * 接收死信的列队,用户监听这个队列
     *
     * @return
     */
    @Bean
    public Queue nftPublishQueue() {
        return new Queue("nft.publish.queue", true);
    }

    /**
     * 死信队列和nft交换机的绑定
     *
     * @return
     */
    @Bean
    public Binding dlNftBinding() {
        return BindingBuilder.bind(dlNftQueue()).to(nftExchange()).with("nft.publish");
    }

    @Bean
    public Binding nftBinding() {
        return BindingBuilder.bind(nftPublishQueue()).to(dlNftExchange()).with("nft");
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }
}
